@@ -1,12 +1,48 @@
+"""
+Rabbitmq related module
+"""
+
 import functools
 import json
+import multiprocessing
+from dataclasses import dataclass, field
 from logging import getLogger
+from multiprocessing.synchronize import Event as EventClass
+from typing import Optional
 
 import pika
+from typing_extensions import TypeAlias
 
+from s3ben.helpers import check_object, remmaping_message
 from s3ben.s3 import S3Events
 
 _logger = getLogger(__name__)
+Queue: TypeAlias = multiprocessing.Queue
+Event: TypeAlias = EventClass
+ConnectionParameters: TypeAlias = pika.ConnectionParameters
+
+
+@dataclass
+class MqConnection:
+    """
+    RabbitMQ dataclass to hold all required parameters in one object
+    """
+
+    host: str
+    user: str
+    password: str = field(repr=False)
+    port: int
+    virtualhost: str
+
+
+@dataclass
+class MqExQue:
+    """
+    Dataclas for Exchange and queue definition
+    """
+
+    exchange: str
+    queue: str
 
 
 class RabbitMQ:
@@ -19,16 +55,8 @@ class RabbitMQ:
     :param str virtualhost: virtual host to connect, default: /
     """
 
-    def __init__(
-        self, hostname: str, user: str, password: str, port: int, virtualhost: str
-    ) -> None:
-        mq_credentials = pika.credentials.PlainCredentials(user, password)
-        self.mq_params = pika.ConnectionParameters(
-            host=hostname,
-            port=port,
-            virtual_host=virtualhost,
-            credentials=mq_credentials,
-        )
+    def __init__(self, conn_params: MqConnection) -> None:
+        self.mq_params = conn_params
         self.should_reconnect: bool = False
         self.was_consuming = False
 
@@ -38,10 +66,12 @@ class RabbitMQ:
         self._closing: bool = False
         self._consumer_tag = None
         self._prefetch_count = 1
-        self._s3_client: S3Events = None
-        self._exchange: str = None
-        self._queue: str = None
-        self._routing_key: str = None
+        self._s3_client: Optional[S3Events] = None
+        self._exchange: Optional[str] = None
+        self._queue: Optional[str] = None
+        self._routing_key: Optional[str] = None
+        self._mp_data_queue: Optional[Queue] = None
+        self._mp_end_event: Optional[Event] = None
 
     def prepare(self, exchange: str, queue: str, routing_key: str) -> None:
         """
@@ -51,7 +81,8 @@ class RabbitMQ:
         :param str routing_key: bind queu to exchange ussing routing key
         """
         _logger.info("Preparing rabbitmq cluster for work")
-        connection = pika.BlockingConnection(parameters=self.mq_params)
+        conn_params = self.__conn_params()
+        connection = pika.BlockingConnection(parameters=conn_params)
         self._channel = connection.channel()
         self.setup_exchange(exchange=exchange)
         self.setup_queue(queue=queue)
@@ -59,7 +90,32 @@ class RabbitMQ:
         self.close_channel()
         connection.close()
 
-    def consume(self, queue: str, s3_client: S3Events) -> None:
+    def __conn_params(self) -> ConnectionParameters:
+        """
+        Method to construct and return pika connection parameters
+        """
+        logins = pika.credentials.PlainCredentials(
+            self.mq_params.user, self.mq_params.password
+        )
+        return pika.ConnectionParameters(
+            host=self.mq_params.host,
+            port=self.mq_params.port,
+            virtual_host=self.mq_params.virtualhost,
+            credentials=logins,
+        )
+
+    def __connect(self):
+        conn_params = self.__conn_params()
+        return pika.SelectConnection(
+            parameters=conn_params,
+            on_open_callback=self.on_connection_open,
+            on_open_error_callback=self.on_connection_open_error,
+            on_close_callback=self.on_connection_closed,
+        )
+
+    def consume(
+        self, queue: str, s3_client: S3Events, mp_data_queue: Queue, mp_end_event: Event
+    ) -> None:
         """
         This method connects to RabbitMQ, returning the connection handle.
         When the connection is established, the on_connection_open method
@@ -67,16 +123,12 @@ class RabbitMQ:
 
         :rtype: pika.SelectConnection
         """
-        _logger.info("Connecting to MQ")
+        self._mp_end_event = mp_end_event
+        self._mp_data_queue = mp_data_queue
         self._s3_client = s3_client
         self._queue = queue
-        self._connection = pika.SelectConnection(
-            parameters=self.mq_params,
-            on_open_callback=self.on_connection_open,
-            on_open_error_callback=self.on_connection_open_error,
-            on_close_callback=self.on_connection_closed,
-        )
-        self._connection.ioloop.start()
+        self._connection = self.__connect()
+        self._connection = self._connection.ioloop.start()
 
     def on_connection_open(self, _unused_connection) -> None:
         """This method is called by pika once the connection to RabbitMQ has
@@ -129,7 +181,7 @@ class RabbitMQ:
         :param pika.channel.Channel: The closed channel
         :param Exception reason: why the channel was closed
         """
-        _logger.warning(f"Channel {channel} was closed: {reason}")
+        _logger.warning("Channel %s was closed: %s", channel, reason)
         self.close_connection()
 
     def close_connection(self) -> None:
@@ -153,7 +205,7 @@ class RabbitMQ:
         :param pika.SelectConnection _unused_connection: The connection
         :param Exception err: The error
         """
-        _logger.error(f"Connection open failed: {err}")
+        _logger.error("Connection open failed: %s", err)
         self.reconnect()
 
     def reconnect(self) -> None:
@@ -209,7 +261,7 @@ class RabbitMQ:
         """
         self._consuming = False
         _logger.debug(
-            f"RabbitMQ acknowledged the cancellation of the consumer: {userdata}"
+            "RabbitMQ acknowledged the cancellation of the consumer: %s", userdata
         )
         self.close_channel()
 
@@ -235,7 +287,7 @@ class RabbitMQ:
         if self._closing:
             self._connection.ioloop.stop()
             return
-        _logger.warning(f"Connection closed, reconnecting because: {reason}")
+        _logger.warning("Connection closed, reconnecting because: %s", reason)
         self.reconnect()
 
     def on_channel_closed(self, channel, reason) -> None:
@@ -249,7 +301,7 @@ class RabbitMQ:
         :param pika.channel.Channel: The closed channel
         :param Exception reason: why the channel was closed
         """
-        _logger.warning(f"Channel {channel} was closed: {reason}")
+        _logger.warning("Channel %s was closed: %s", channel, reason)
         self.close_connection()
 
     def setup_queue(self, queue) -> None:
@@ -259,7 +311,7 @@ class RabbitMQ:
 
         :param str|unicode queue_name: The name of the queue to declare.
         """
-        _logger.debug(f"Creating queue: {queue}")
+        _logger.debug("Creating queue: %s", queue)
         self._queue = queue
         self._channel.queue_declare(
             queue=self._queue,
@@ -269,7 +321,12 @@ class RabbitMQ:
         )
 
     def quene_bind(self, routing_key: str) -> None:
-        _logger.debug(f"Binding {self._exchange} to {self._queue} with {routing_key}")
+        """
+        Method to bind queue and exchange toggether.
+        """
+        _logger.debug(
+            "Binding %s to %s with %s", self._exchange, self._queue, routing_key
+        )
         self._channel.queue_bind(
             queue=self._queue, exchange=self._exchange, routing_key=routing_key
         )
@@ -281,7 +338,7 @@ class RabbitMQ:
 
         :param str|unicode exchange_name: The name of the exchange to declare
         """
-        _logger.debug(f"Declaring exhange {exchange}")
+        _logger.debug("Declaring exhange %s", exchange)
         self._exchange = exchange
         self._channel.exchange_declare(
             exchange=self._exchange,
@@ -322,11 +379,17 @@ class RabbitMQ:
 
         :param pika.frame.Method method_frame: The Basic.Cancel frame
         """
-        _logger.info(f"Consumer canceled, shuting down {method_frame}")
+        _logger.info("Consumer canceled, shuting down %s", method_frame)
         if self._channel:
             self._channel.close()
 
-    def on_message(self, _unused_channel, basic_deliver, properties, body) -> None:
+    def on_message(
+        self,
+        _unused_channel,
+        basic_deliver,
+        properties,
+        body,
+    ) -> None:
         """
         Invoked by pika when a message is delivered from RabbitMQ. The
         channel is passed for your convenience. The basic_deliver object that
@@ -349,15 +412,62 @@ class RabbitMQ:
             bucket = record["s3"]["bucket"]["name"]
             obj_key = record["s3"]["object"]["key"]
             _logger.debug(
-                f"Bucket: {bucket} event: {event} file: {obj_key} size: {obj_size}"
+                "Bucket: %s event: %s file: %s size: %s",
+                bucket,
+                event,
+                obj_key,
+                obj_size,
             )
             action = self.__bucket_event(event=event)
+            key = check_object(obj_key)
             if action == "download":
-                self._s3_client.download_object(bucket=bucket, path=obj_key)
+                if isinstance(key, dict):
+                    self._download_remapped(bucket=bucket, data=key)
+                else:
+                    _ = self._s3_client.download_object_v2(  # type: ignore[union-attr]
+                        bucket=bucket, s3_obj=key, local_path=key
+                    )
+
             if action == "remove":
-                self._s3_client.remove_object(bucket=bucket, path=obj_key)
+                if isinstance(key, dict):
+                    self._remove_remapped(bucket=bucket, data=key)
+                else:
+                    self._s3_client.remove_object(  # type: ignore[union-attr]
+                        bucket=bucket, path=obj_key
+                    )
         self.acknowledge_message(basic_deliver.delivery_tag)
 
+    def _remove_remapped(self, bucket, data: dict) -> None:
+        """
+        Remove remmaped object from active and database
+
+        :param str bucket: Bucket name
+        :param dict data: Dictionary containing remapping info
+        :rtype: None
+        """
+        _, local_path, message = remmaping_message(
+            action="remove", remap=data, bucket=bucket
+        )
+        self._s3_client.remove_object(bucket=bucket, path=local_path)  # type: ignore[union-attr]
+        self._mp_data_queue.put_nowait(message)  # type: ignore[union-attr]
+
+    def _download_remapped(self, bucket: str, data: dict) -> None:
+        """
+        Method to download and remap object
+
+        :param str bucket: Bucket name
+        :param dict data: Dictionary containing remapping info
+        :rtype: None
+        """
+        s3_obj, local_path, message = remmaping_message(
+            action="download", remap=data, bucket=bucket
+        )
+        dl_status = self._s3_client.download_object_v2(  # type: ignore[union-attr]
+            bucket=bucket, s3_obj=s3_obj, local_path=local_path
+        )
+        if dl_status:
+            self._mp_data_queue.put_nowait(message)  # type: ignore[union-attr]
+
     def __bucket_event(self, event: str) -> str:
         """
         Method to parse bucket event
@@ -385,7 +495,7 @@ class RabbitMQ:
 
         :param int delivery_tag: The delivery tag from the Basic.Deliver frame
         """
-        _logger.debug(f"Ack message: {delivery_tag}")
+        _logger.debug("Ack message: %s", delivery_tag)
         self._channel.basic_ack(delivery_tag)
 
     def on_basic_qos_ok(self, _unused_frame):
@@ -406,7 +516,7 @@ class RabbitMQ:
         with different prefetch values to achieve desired performance.
 
         """
-        _logger.debug(f"Setting prefetch: {self._prefetch_count}")
+        _logger.debug("Setting prefetch: %s", self._prefetch_count)
         self._channel.basic_qos(
             prefetch_count=self._prefetch_count, callback=self.on_basic_qos_ok
         )
